
#ifndef VTKPVVTKEXTENSIONSRENDERING_EXPORT_H
#define VTKPVVTKEXTENSIONSRENDERING_EXPORT_H

#ifdef VTKPVVTKEXTENSIONSRENDERING_STATIC_DEFINE
#  define VTKPVVTKEXTENSIONSRENDERING_EXPORT
#  define VTKPVVTKEXTENSIONSRENDERING_NO_EXPORT
#else
#  ifndef VTKPVVTKEXTENSIONSRENDERING_EXPORT
#    ifdef VTKExtensionsRendering_EXPORTS
        /* We are building this library */
#      define VTKPVVTKEXTENSIONSRENDERING_EXPORT __declspec(dllexport)
#    else
        /* We are using this library */
#      define VTKPVVTKEXTENSIONSRENDERING_EXPORT __declspec(dllimport)
#    endif
#  endif

#  ifndef VTKPVVTKEXTENSIONSRENDERING_NO_EXPORT
#    define VTKPVVTKEXTENSIONSRENDERING_NO_EXPORT 
#  endif
#endif

#ifndef VTKPVVTKEXTENSIONSRENDERING_DEPRECATED
#  define VTKPVVTKEXTENSIONSRENDERING_DEPRECATED __declspec(deprecated)
#endif

#ifndef VTKPVVTKEXTENSIONSRENDERING_DEPRECATED_EXPORT
#  define VTKPVVTKEXTENSIONSRENDERING_DEPRECATED_EXPORT VTKPVVTKEXTENSIONSRENDERING_EXPORT VTKPVVTKEXTENSIONSRENDERING_DEPRECATED
#endif

#ifndef VTKPVVTKEXTENSIONSRENDERING_DEPRECATED_NO_EXPORT
#  define VTKPVVTKEXTENSIONSRENDERING_DEPRECATED_NO_EXPORT VTKPVVTKEXTENSIONSRENDERING_NO_EXPORT VTKPVVTKEXTENSIONSRENDERING_DEPRECATED
#endif

#if 0 /* DEFINE_NO_DEPRECATED */
#  ifndef VTKPVVTKEXTENSIONSRENDERING_NO_DEPRECATED
#    define VTKPVVTKEXTENSIONSRENDERING_NO_DEPRECATED
#  endif
#endif

#endif /* VTKPVVTKEXTENSIONSRENDERING_EXPORT_H */
