"""This is present for backwards compatibility alone. Scripts are encouraged to
use `vtkConstants` from `vtk` package directly as follows:

    from paraview.vtk.util import vtkConstants

"""
from vtkmodules.util.vtkConstants import *
