
#ifndef VTKPVANIMATION_EXPORT_H
#define VTKPVANIMATION_EXPORT_H

#ifdef VTKPVANIMATION_STATIC_DEFINE
#  define VTKPVANIMATION_EXPORT
#  define VTKPVANIMATION_NO_EXPORT
#else
#  ifndef VTKPVANIMATION_EXPORT
#    ifdef Animation_EXPORTS
        /* We are building this library */
#      define VTKPVANIMATION_EXPORT __declspec(dllexport)
#    else
        /* We are using this library */
#      define VTKPVANIMATION_EXPORT __declspec(dllimport)
#    endif
#  endif

#  ifndef VTKPVANIMATION_NO_EXPORT
#    define VTKPVANIMATION_NO_EXPORT 
#  endif
#endif

#ifndef VTKPVANIMATION_DEPRECATED
#  define VTKPVANIMATION_DEPRECATED __declspec(deprecated)
#endif

#ifndef VTKPVANIMATION_DEPRECATED_EXPORT
#  define VTKPVANIMATION_DEPRECATED_EXPORT VTKPVANIMATION_EXPORT VTKPVANIMATION_DEPRECATED
#endif

#ifndef VTKPVANIMATION_DEPRECATED_NO_EXPORT
#  define VTKPVANIMATION_DEPRECATED_NO_EXPORT VTKPVANIMATION_NO_EXPORT VTKPVANIMATION_DEPRECATED
#endif

#if 0 /* DEFINE_NO_DEPRECATED */
#  ifndef VTKPVANIMATION_NO_DEPRECATED
#    define VTKPVANIMATION_NO_DEPRECATED
#  endif
#endif

#endif /* VTKPVANIMATION_EXPORT_H */
