
#ifndef VTKPVSERVERMANAGERDEFAULT_EXPORT_H
#define VTKPVSERVERMANAGERDEFAULT_EXPORT_H

#ifdef VTKPVSERVERMANAGERDEFAULT_STATIC_DEFINE
#  define VTKPVSERVERMANAGERDEFAULT_EXPORT
#  define VTKPVSERVERMANAGERDEFAULT_NO_EXPORT
#else
#  ifndef VTKPVSERVERMANAGERDEFAULT_EXPORT
#    ifdef ServerManagerDefault_EXPORTS
        /* We are building this library */
#      define VTKPVSERVERMANAGERDEFAULT_EXPORT __declspec(dllexport)
#    else
        /* We are using this library */
#      define VTKPVSERVERMANAGERDEFAULT_EXPORT __declspec(dllimport)
#    endif
#  endif

#  ifndef VTKPVSERVERMANAGERDEFAULT_NO_EXPORT
#    define VTKPVSERVERMANAGERDEFAULT_NO_EXPORT 
#  endif
#endif

#ifndef VTKPVSERVERMANAGERDEFAULT_DEPRECATED
#  define VTKPVSERVERMANAGERDEFAULT_DEPRECATED __declspec(deprecated)
#endif

#ifndef VTKPVSERVERMANAGERDEFAULT_DEPRECATED_EXPORT
#  define VTKPVSERVERMANAGERDEFAULT_DEPRECATED_EXPORT VTKPVSERVERMANAGERDEFAULT_EXPORT VTKPVSERVERMANAGERDEFAULT_DEPRECATED
#endif

#ifndef VTKPVSERVERMANAGERDEFAULT_DEPRECATED_NO_EXPORT
#  define VTKPVSERVERMANAGERDEFAULT_DEPRECATED_NO_EXPORT VTKPVSERVERMANAGERDEFAULT_NO_EXPORT VTKPVSERVERMANAGERDEFAULT_DEPRECATED
#endif

#if 0 /* DEFINE_NO_DEPRECATED */
#  ifndef VTKPVSERVERMANAGERDEFAULT_NO_DEPRECATED
#    define VTKPVSERVERMANAGERDEFAULT_NO_DEPRECATED
#  endif
#endif

#endif /* VTKPVSERVERMANAGERDEFAULT_EXPORT_H */
