
#ifndef VTKPVCLIENTSERVERCOREDEFAULT_EXPORT_H
#define VTKPVCLIENTSERVERCOREDEFAULT_EXPORT_H

#ifdef VTKPVCLIENTSERVERCOREDEFAULT_STATIC_DEFINE
#  define VTKPVCLIENTSERVERCOREDEFAULT_EXPORT
#  define VTKPVCLIENTSERVERCOREDEFAULT_NO_EXPORT
#else
#  ifndef VTKPVCLIENTSERVERCOREDEFAULT_EXPORT
#    ifdef ClientServerCoreDefault_EXPORTS
        /* We are building this library */
#      define VTKPVCLIENTSERVERCOREDEFAULT_EXPORT __declspec(dllexport)
#    else
        /* We are using this library */
#      define VTKPVCLIENTSERVERCOREDEFAULT_EXPORT __declspec(dllimport)
#    endif
#  endif

#  ifndef VTKPVCLIENTSERVERCOREDEFAULT_NO_EXPORT
#    define VTKPVCLIENTSERVERCOREDEFAULT_NO_EXPORT 
#  endif
#endif

#ifndef VTKPVCLIENTSERVERCOREDEFAULT_DEPRECATED
#  define VTKPVCLIENTSERVERCOREDEFAULT_DEPRECATED __declspec(deprecated)
#endif

#ifndef VTKPVCLIENTSERVERCOREDEFAULT_DEPRECATED_EXPORT
#  define VTKPVCLIENTSERVERCOREDEFAULT_DEPRECATED_EXPORT VTKPVCLIENTSERVERCOREDEFAULT_EXPORT VTKPVCLIENTSERVERCOREDEFAULT_DEPRECATED
#endif

#ifndef VTKPVCLIENTSERVERCOREDEFAULT_DEPRECATED_NO_EXPORT
#  define VTKPVCLIENTSERVERCOREDEFAULT_DEPRECATED_NO_EXPORT VTKPVCLIENTSERVERCOREDEFAULT_NO_EXPORT VTKPVCLIENTSERVERCOREDEFAULT_DEPRECATED
#endif

#if 0 /* DEFINE_NO_DEPRECATED */
#  ifndef VTKPVCLIENTSERVERCOREDEFAULT_NO_DEPRECATED
#    define VTKPVCLIENTSERVERCOREDEFAULT_NO_DEPRECATED
#  endif
#endif

#endif /* VTKPVCLIENTSERVERCOREDEFAULT_EXPORT_H */
