
#ifndef VTKPVCLIENTSERVERCORERENDERING_EXPORT_H
#define VTKPVCLIENTSERVERCORERENDERING_EXPORT_H

#ifdef VTKPVCLIENTSERVERCORERENDERING_STATIC_DEFINE
#  define VTKPVCLIENTSERVERCORERENDERING_EXPORT
#  define VTKPVCLIENTSERVERCORERENDERING_NO_EXPORT
#else
#  ifndef VTKPVCLIENTSERVERCORERENDERING_EXPORT
#    ifdef ClientServerCoreRendering_EXPORTS
        /* We are building this library */
#      define VTKPVCLIENTSERVERCORERENDERING_EXPORT __declspec(dllexport)
#    else
        /* We are using this library */
#      define VTKPVCLIENTSERVERCORERENDERING_EXPORT __declspec(dllimport)
#    endif
#  endif

#  ifndef VTKPVCLIENTSERVERCORERENDERING_NO_EXPORT
#    define VTKPVCLIENTSERVERCORERENDERING_NO_EXPORT 
#  endif
#endif

#ifndef VTKPVCLIENTSERVERCORERENDERING_DEPRECATED
#  define VTKPVCLIENTSERVERCORERENDERING_DEPRECATED __declspec(deprecated)
#endif

#ifndef VTKPVCLIENTSERVERCORERENDERING_DEPRECATED_EXPORT
#  define VTKPVCLIENTSERVERCORERENDERING_DEPRECATED_EXPORT VTKPVCLIENTSERVERCORERENDERING_EXPORT VTKPVCLIENTSERVERCORERENDERING_DEPRECATED
#endif

#ifndef VTKPVCLIENTSERVERCORERENDERING_DEPRECATED_NO_EXPORT
#  define VTKPVCLIENTSERVERCORERENDERING_DEPRECATED_NO_EXPORT VTKPVCLIENTSERVERCORERENDERING_NO_EXPORT VTKPVCLIENTSERVERCORERENDERING_DEPRECATED
#endif

#if 0 /* DEFINE_NO_DEPRECATED */
#  ifndef VTKPVCLIENTSERVERCORERENDERING_NO_DEPRECATED
#    define VTKPVCLIENTSERVERCORERENDERING_NO_DEPRECATED
#  endif
#endif

#endif /* VTKPVCLIENTSERVERCORERENDERING_EXPORT_H */
