#ifndef vtk_gl2ps_mangle_h
#define vtk_gl2ps_mangle_h

#define gl2psAddPolyPrimitive vtkgl2ps_gl2psAddPolyPrimitive
#define gl2psBeginPage vtkgl2ps_gl2psBeginPage
#define gl2psBeginViewport vtkgl2ps_gl2psBeginViewport
#define gl2psBlendFunc vtkgl2ps_gl2psBlendFunc
#define gl2psDisable vtkgl2ps_gl2psDisable
#define gl2psDrawImageMap vtkgl2ps_gl2psDrawImageMap
#define gl2psDrawPixels vtkgl2ps_gl2psDrawPixels
#define gl2psEnable vtkgl2ps_gl2psEnable
#define gl2psEndPage vtkgl2ps_gl2psEndPage
#define gl2psEndViewport vtkgl2ps_gl2psEndViewport
#define gl2psForceRasterPos vtkgl2ps_gl2psForceRasterPos
#define gl2psGetFileExtension vtkgl2ps_gl2psGetFileExtension
#define gl2psGetFileFormat vtkgl2ps_gl2psGetFileFormat
#define gl2psGetFormatDescription vtkgl2ps_gl2psGetFormatDescription
#define gl2psGetOptions vtkgl2ps_gl2psGetOptions
#define gl2psLineCap vtkgl2ps_gl2psLineCap
#define gl2psLineJoin vtkgl2ps_gl2psLineJoin
#define gl2psLineWidth vtkgl2ps_gl2psLineWidth
#define gl2psPointSize vtkgl2ps_gl2psPointSize
#define gl2psSetOptions vtkgl2ps_gl2psSetOptions
#define gl2psSpecial vtkgl2ps_gl2psSpecial
#define gl2psSpecialColor vtkgl2ps_gl2psSpecialColor
#define gl2psText vtkgl2ps_gl2psText
#define gl2psTextOpt vtkgl2ps_gl2psTextOpt
#define gl2psTextOptColor vtkgl2ps_gl2psTextOptColor
#define gl2psTextOptColorBL vtkgl2ps_gl2psTextOptColorBL

#endif
